using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using MikuMikuDance.XNA;
using MikuMikuDance.XNA.Model;
using MikuMikuDance.XNA.Motion;
using System.Diagnostics;
using MikuMikuDance.XNA.Accessory;

namespace MikuMikuDanceXNADemo5
{
    /// <summary>
    /// MikuMikuDance for XNAのデモ
    /// このデモでは歩きモーションデモです
    /// DirectXのスキニングメッシュサンプルっぽいものが流れます
    /// モーションループ再生時の動作フラグの利用法の一つが書かれています
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        //MikuMikuDance for XNA
        MikuMikuDanceXNA mmd;
        //MMDモデルデータ
        MMDModel model;
        //MMDモーションデータ
        MMDBakedMotion motion;
        //MMDアクセサリデータ
        MMDAccessory negi;
        //ストップウォッチ
        Stopwatch stopwatch = new Stopwatch();
        public Game1()
        {
            //デバイスがMMDXの必要条件を満たしているかチェック
            graphics = new GraphicsDeviceManager(this);
            MikuMikuDanceXNA.CheckDevice(graphics);
            Content.RootDirectory = "Content";
        }

        protected override void Initialize()
        {
            base.Initialize();
        }

        /// <summary>
        /// LoadContent関数
        /// 各種ロード処理はここでロードを行っておくと良い
        /// </summary>
        protected override void LoadContent()
        {
            //MikuMikuDanceXNAを作成する
            mmd = new MikuMikuDanceXNA(this);
            //プロジェクトのContent内に追加してあるMiku.pmdモデルをロードする
            //ロードはMikuMikuDanceXNA.LoadModel関数で行う
            //(モデルデータはあにまさ氏よりお借りしました。このモデルファイルはMITライセンス適応外です)
            model = mmd.LoadModel("Miku", this, Matrix.Identity);
            //同じくContent内に追加してあるnegi.xアクセサリをロードする
            //(アクセサリデータはあにまさ氏よりお借りしました。このモデルファイルはMITライセンス適応外です)
            negi = mmd.LoadAccessory("negi", this, Matrix.Identity);//プロセッサの設定で14倍拡大してある
            //アクセサリを持たせる用のnegi-vac.vacをロードする
            MMD_VAC vac = mmd.LoadVAC("negi-vac");
            //モデルにVACを指定することで持たせる(Matrixでも指定可)
            model.SetAccessory(negi, vac);

            //同じくContent内に追加してあるTrueMyHeart.vmdモーションをロードする
            //ロードはMikuMikuDanceXNA.LoadMotion関数で行う
            motion = mmd.LoadBakedMotion("WalkB");//あまりうまく歩かせれなかった……モーション製作の問題
            //モデルのプレイヤーを呼び出し
            //モーションを0番トラックにセットする
            model.Player.SetMotion(0, motion, true);
            model.Player.FramePerSecond = 60;//倍速再生する(通常FPSは30)

            mmd.Camera.CameraPos = new Vector3(-20, 20, 23);
            mmd.Camera.CameraTarget = new Vector3(0, 10, 3);
        }

        /// <summary>
        /// UnloadContent関数
        /// アンロードが必要な物はここでアンロードを行う
        /// </summary>
        protected override void UnloadContent()
        {
        }

        /// <summary>
        /// アップデート関数
        /// 毎フレーム処理する描画以外のコードをここに記述する
        /// </summary>
        /// <param name="gameTime">現在の時間のスナップショットを表すオブジェクト</param>
        protected override void Update(GameTime gameTime)
        {
            //XBoxコントローラーのBackボタンを押したら終了
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();
            //キーボードのEscキーを押したら終了
            if (Keyboard.GetState().IsKeyDown(Keys.Escape))
                this.Exit();

            //現在再生中でなければ
            if (!model.Player.IsPlay(0))
            {
                //0番トラックにセットされているモーションをリセット
                model.Player.Reset(0);
                //0番トラックにセットされているモーションを再生する
                //ループごとににセンターボーンのZ軸のズレでモデルのワールド座標系を修正する。
                //それにより、うまくループさせられる……はず。(モーション製作があまりうまくないです。はい。)
                //(こういうループ再生の利用をするなら、センターボーンの位置は変えずに、プログラム側でモーションに合わせてワールド座標系を修正するルーチン書いた方がいいかもしれない……)
                model.Player.Start(0, true, RepeatBehaviorFlag.ModifyByZ);
                //ストップウォッチ開始
                stopwatch.Start();
            }
            mmd.Camera.CameraPos = new Vector3(-20, 20, 23f + (float)stopwatch.ElapsedMilliseconds / 102.5f);
            mmd.Camera.CameraTarget = new Vector3(0, 10, 3f + (float)stopwatch.ElapsedMilliseconds / 102.5f);

            //モデルの再生処理はDrawableGameComponentを継承しているため
            //XNAにより自動で処理が行われる

            //XNA内部のUpdate処理を行う
            base.Update(gameTime);
        }

        /// <summary>
        /// Draw関数
        /// 毎フレーム処理する描画コードをここに記述する
        /// </summary>
        /// <param name="gameTime">現在の時間のスナップショットを表すオブジェクト</param>
        protected override void Draw(GameTime gameTime)
        {
            //画面をCornflowerBlue色で消去する
            GraphicsDevice.Clear(Color.CornflowerBlue);

            //モデルはDrawableGameComponentを継承しているため
            //XNAにより自動で描画される

            //XNA内部のDraw処理を行う
            base.Draw(gameTime);
        }
    }
}
